﻿using System;
using System.Drawing;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace TopMoster
{
	public partial class MainForm : Form
	{
		#region 【Win32API】

		/// <summary>
		/// 【構造体】 POINT(位置)
		/// </summary>
		[StructLayout(LayoutKind.Sequential)]
		public struct POINT
		{
			public int x;
			public int y;
		}

		/// <summary>
		/// SetWindowPosの引数フラグ（Zオーダー）
		/// </summary>
		private enum SWPZORDER: int
		{
			HWND_TOPMOST = -1,	// 最前面
			HWND_TOP = 0,		// 前面
			HWND_NOTOPMOST = -2,// すべての最前面ウィンドウの後ろ
			HWND_BOTTOM = 1,	// Z オーダーの最後
		}
		/// <summary>
		/// SetWindowPosの引数フラグ（位置サイズ）
		/// </summary>
		private enum SWP : int
		{
			SWP_NOSIZE = 0x0001,	// 現在のサイズを維持（cx、cyは無視）
			SWP_NOMOVE = 0x0002,	// 現在の位置を維持します（x、yは無視）
			SWP_NOZORDER = 0x0004,
			SWP_NOREDRAW = 0x0008,
			SWP_NOACTIVATE = 0x0010,

			SWP_FRAMECHANGED = 0x0020,
			SWP_SHOWWINDOW = 0x0040,	// ウィンドウを表示
			SWP_HIDEWINDOW = 0x0080,

			SWP_NOCOPYBITS = 0x0100,
			SWP_NOOWNERZORDER = 0x0200,
			SWP_NOSENDCHANGING = 0x400,

		}
		/// <summary>
		/// GetAncestorの引数フラグ（親ウィンドウ取得フラグ）
		/// </summary>
		private enum GA : uint
		{
			GA_PARENT = 1,		// 親ウィンドウを返す
			GA_ROOT = 2,		// 親子関係を遡って、直近上位のトップレベルウィンドウを返す
			GA_ROOTOWNER = 3,	// 親子関係と所有関係を遡って、所有されていないトップレベルウィンドウを返す

		}

		/// <summary>
		/// 【Win32API】ウィンドウのサイズ、位置、および Z オーダーを変更します
		/// </summary>
		/// <param name="hWnd">対象ウィンドウ</param>
		/// <param name="hWndInsertAfter">Zオーダー</param>
		/// <param name="x">X座標</param>
		/// <param name="y">Y座標</param>
		/// <param name="width">横幅</param>
		/// <param name="height">高さ</param>
		/// <param name="flags">変更フラグ</param>
		/// <returns></returns>
		[DllImport("user32.dll")]
		private static extern UInt32 SetWindowPos(IntPtr hWnd,
			SWPZORDER hWndInsertAfter,
			int x, int y, int width, int height, SWP flags);
	
		/// <summary>
		/// 指定座標からウインドウのハンドルを取得する。
		/// </summary>
		/// <param name="Point">指定座標</param>
		/// <returns>ウィンドウハンドル</returns>
		[DllImport("user32.dll")]
		private static extern IntPtr WindowFromPoint(POINT Point);

		/// <summary>
		/// 親ウインドウのハンドルを取得。
		/// </summary>
		/// <param name="hwnd">対象ウィンドウ</param>
		/// <param name="gaFlags">検索フラグ</param>
		/// <returns>親ウインドウのハンドル</returns>
		[DllImport("user32.dll")]
		private static extern IntPtr GetAncestor(IntPtr hwnd, GA gaFlags);

		#endregion 【Win32API】

		/// <summary>
		/// コンストラクタ
		/// </summary>
		public MainForm()
		{
			InitializeComponent();
		}

		/// <summary>
		/// 【ロジック】Zオーダー変更
		/// </summary>
		/// <param name="swpZorder">Zオーダーフラグ</param>
		private void SetZOrder(SWPZORDER swpZorder)
		{
			// マウスカーソル座標取得
			Point po = Cursor.Position;

			// Win32APIを呼ぶためにPoint型をPOINT構造体にセット
			POINT poStruct;
			poStruct.x = po.X;
			poStruct.y = po.Y;

			// マウスの座標にあるウィンドウのハンドルを取得
			IntPtr hWnd = WindowFromPoint(poStruct);
			// ウィンドウハンドルが正常に取得できれば
			if (hWnd != null)
			{
				// 指定ウィンドウのオーナーを取得
				IntPtr hWndRoot = GetAncestor(hWnd, GA.GA_ROOT);
				// オーナーのウィンドウハンドルが取得できれば
				if (hWndRoot != null)
				{
					// Zオーダー変更の実行
					SetWindowPos(hWndRoot, swpZorder, 0, 0, 0, 0, SWP.SWP_NOMOVE | SWP.SWP_NOSIZE);
				}
			}
		}

		/// <summary>
		/// 【イベント】最前面化ラベル マウス押上イベント
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void ctLblTopMost_MouseUp(object sender, MouseEventArgs e)
		{
			// Zオーダー変更 最前面化
			this.SetZOrder(SWPZORDER.HWND_TOPMOST);
		}

		/// <summary>
		/// 【イベント】非 最前面化ラベル マウス押上イベント
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void ctLblNotTopMost_MouseUp(object sender, MouseEventArgs e)
		{
			// Zオーダー変更 非 最前面化
			this.SetZOrder(SWPZORDER.HWND_NOTOPMOST);
		}

	}
}
